/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "XPathExpression.h"
#include "XPathResult.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomXPathExpression.h"
#include "gdom/GdomXPathResult.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateGdomXPathExpression.h"
#include "gdom/PrivateGdomXPathResult.h"
#include "gdom/PrivateIntGdomNode.h"
#include "gdom/PrivateIntGdomXPathExpression.h"
#include "gdom/PrivateIntGdomXPathResult.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::XPathExpression* obj)
{
    return WebCore::cacheDOMObject<WebCore::XPathExpression, GDOMXPathExpression>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMXPathExpression::GDOMXPathExpression(PassRefPtr<WebCore::XPathExpression> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMXPathExpression::~GDOMXPathExpression()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

GdomXPathResult *
gdom_x_path_expression_evaluate (GdomXPathExpression *thiz, GdomNode * context_node, gushort type, GdomXPathResult * in_result)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::XPathExpression * item = (gdom_x_path_expression_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (context_node, 0);
    g_return_val_if_fail (in_result, 0);
    WebCore::Node * _g_context_node = (gdom_node_get_private(context_node));
    g_return_val_if_fail (_g_context_node, 0);
    WebCore::XPathResult * _g_in_result = (gdom_x_path_result_get_private(in_result));
    g_return_val_if_fail (_g_in_result, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::XPathResult> g_res = WTF::getPtr(item->evaluate(_g_context_node, type, _g_in_result, ec));
    GdomXPathResult * res = static_cast<GdomXPathResult *>(WebCore::toGDOM(g_res.get()));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "XPathExpression.h"
#include <gdom/GdomXPathExpression.h>
#include <gdom/PrivateGdomXPathExpression.h>
#include <gdom/PrivateIntGdomXPathExpression.h>
#include <gdom/GdomXPathExpressionprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomXPathExpression, gdom_x_path_expression, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomXPathExpressionPrivate {
    class WebCore::XPathExpression *item;
};

#define GDOM_X_PATH_EXPRESSION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_X_PATH_EXPRESSION, GdomXPathExpressionPrivate))
*/
G_END_DECLS
WebCore::XPathExpression *
gdom_x_path_expression_get_private(GdomXPathExpression *request)
{
    GDOMXPathExpression *res;
    res = static_cast<GDOMXPathExpression*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::XPathExpression*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
};


static void gdom_x_path_expression_finalize(GObject* object)
{
    /*GdomXPathExpression* request = GDOM_X_PATH_EXPRESSION(object);*/
    /*GdomXPathExpressionPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_x_path_expression_parent_class)->finalize(object);
}

static void gdom_x_path_expression__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomXPathExpression *self = GDOM_X_PATH_EXPRESSION(object);
    WebCore::XPathExpression * gself = (gdom_x_path_expression_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_x_path_expression__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomXPathExpression *self = GDOM_X_PATH_EXPRESSION(object);
    WebCore::XPathExpression * gself = (gdom_x_path_expression_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_x_path_expression_class_init(GdomXPathExpressionClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_x_path_expression_finalize;
    g_klass->set_property = gdom_x_path_expression__set_property_fn;
    g_klass->get_property = gdom_x_path_expression__get_property_fn;





    /*g_type_class_remove_private(requestClass, sizeof(GdomXPathExpressionPrivate));*/
}

static void gdom_x_path_expression_init(GdomXPathExpression* request)
{
    /*GdomXPathExpressionPrivate* priv = GDOM_X_PATH_EXPRESSION_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

